CON
{{The CON section defines the constants used within the software.}} 
  _clkmode = xtal1 + pll16x                                                             'Set up frequency and clock data
  _xinfreq = 5_000_000
                                                                                        ' These are enumerations:
  CR = 62                                                                               ' ASCII <CR>
  LF = 10                                                                               ' ASCII <LF>
  comma = 44                                                                            ' Comma

  serXmit   =  8 '''                                                                       ' serXmit is the pin on which GPS data could be sent                   
  serRecv   =  9 '''                                                                       ' serRecv is the pin on which GPS data is received
  
  LCD_PIN   = 16 '''                                                                        ' For AMC1602AR serial LCD on pin 23
  LCD_BAUD  = 19_200                                                                    ' Baud rate for communication to AMC1602AR LCD is set
  LCD_LINES = 2                                                                         ' The LCD has two lines
  
 
  LED1_PIN = 20 '''                                                                        ' LED1 is connected to pin 18
  LED2_PIN = 21 '''                                                                        ' LED2 is connected to pin 19
  SW1_PIN = 19 '''                                                                           ' Switch one ("Left") is connected to pin 4
  SW2_PIN = 18 '''                                                                           ' Switch two ("Right") is connected to pin 5
  SW3_PIN = 17 '''                                                                           ' Switch three ("Record") is connected to pin 6
  ELM_BAUD = 38400                                                                      ' The baud rate to the ELM327
 
  xAxis = 0                                                                             ' The X-axis of the accelerometer is on ADC channel 0
  yAxis = 1                                                                             ' The Y-axis of the accelerometer is on ADC channel 1
  zAxis = 2                                                                             ' The Z-axis of the accelerometer is on ADC channel 2
  VRef = 3                                                                              ' VRef of the ADC is on pin 3
  yRate = 4
  xRate = 5
  supply = 6                                                                            ' The supply voltage of the ADC is on pin 6
 
  DIO = 1                                                                              ' The DIO for the ADC is on pin 30
  CLK = 0                                                                              ' The CLK for the ADC is on pin 29
  CS = 2                                                                               ' The CS for the ADC is on pin 31
 
                                                                            ' Whether GPS is included (1=Y, 0=N)
  doErr = 1                                                                             ' Whether we wish to report comms errors

  Selected = 1                                                                          ' Enumeration of switch position
  NotSelected = 0                                                                       ' Enumeration of switch position
  
  No = 0                                                                                ' Enumeration of 'No'
  Yes =1                                                                                ' Enumeration of 'Yes'
OBJ
{{The OBJ section defines the objects that are used within the software.}} 

  uart          :               "FullDuplexSerial"                                      ' This object communicates with the ELM327
  lcd           :               "Debug_Lcd_AMC1602"                                     ' This is the LCD display
  sdfat         :               "fsrw"                                                  ' This is the SD card on which data is recorded
  num           :               "simple_numbers"                                        ' This object handles simple numerical operations
  flt           :               "FloatMath"                                             ' This object deaks with the complex floating-point math
  'ADC           :               "MCP3208"                                               ' This is the ADC object used to deal with the accelerometer data
  fltstr        :               "FloatString"                                           ' This object will be used to convert floating-point numbers to strings
 

VAR
{{The VAR section declares the global variables across the software.}}
Byte Rx                                                                                 ' Single byte for received data
Byte dataIn[100]                                                                        ' Array for data received from ELM327
Byte ct                                                                                 ' Number of bytes received from ELM327
Byte crt                                                                                ' Counter
Byte lcdShow                                                                            ' Whether or not to show data. 1=Yes, 0=No
Byte H3, H4                                                                             ' characters of hex data returned from ELM327 PID Data
Byte lcdMode                                                                            ' Whether data being displayed should be treated as a string or a decimal
Byte rec                                                                                ' Recording mode. 0=Not recording; 1=recording OBD data; 2=recording acc
Byte index1                                                                             ' Store for switch data
Byte mode                                                                               ' The type of OBD data being displayed
Byte maxmode, dataOut[10]                                                                            ' The maximum number of OBD types available

long txDisp                                                                             ' The title text to be displayed with OBD data
long units                                                                              ' The 'units' to be displayed after the OBD data
long AnsaB, AnsaA, AnsaC, q                                                             ' Answers from interpreting the OBD data
long stack1[10]
byte r
byte sw1, sw2                                                                           ' States of switches 1 & 2: 1=Depressed; 0=Not depressed
byte err                                                                                ' Error detected; whether to ignore ELM327 wngs: 1=catch errors
long aq, aqin                                                                           ' Accelerometer data
byte charCtr, strptr

pub main   
{{Pub MAIN is the initial procedure called. Various objects are initialised for future use and inputs and outputs initialised. Having
made connection to the ELM327 the procedure presents the user with an option as to whether to review fault data or to look at live data.}}

'1) Set up pins
'==============
dira[LED1_PIN]~~                                                                        ' Set the pin driving LED1 to output
dira[LED2_PIN]~~                                                                        ' Set the pin driving LED2 to output
dira[SW1_PIN]~                                                                          ' Set the pin monitoring switch 1 to input
dira[SW2_PIN]~                                                                          ' Set the pin monitoring switch 2 to input
dira[SW3_PIN]~                                                                          ' Set the pin monitoring switch 3 to input

'2) Initialise variables
'=======================
rec := 0                                                                                ' The default recording state is NOT to record
mode := 11                                                                               ' The initial OBD data mode is 11 (acceleration data)
maxmode := 12                                                                           ' This is the maximum number of 'modes'


'3) Start Objects
'================
'3.1) Start the ADC
'------------------
'ADC.start(DIO, CLK, CS, %11100000)                                                      ' Start the ADC object handling accelerometer data


'3.2) Start the RS232 comms with the ELM327
'------------------------------------------
uart.start(serRecv, serXmit, 3, ELM_BAUD)                                               ' Start the [two-way] RS232 communications with ELM327
uart.rxflush                                                                            ' Flush the buffer       

'3.3) Start the LCD Display
'--------------------------
lcd.init( LCD_PIN,LCD_BAUD,LCD_LINES)                                                   ' Start the LCD display
callCls                                                                                 ' Clear the display
callPutC(254)
callPutC(104)

'3.4) Start the SD card
'----------------------
lcd.str(string("Mounting SD..."))                                                       ' Tell the user what is happening
r:=  sdfat.mount(3)                                                                     ' Mount the drive (based on pin 0) and receive a number wih the result
lcd.dec(r)                                                                              ' Display (briefly) the result
callCls
if r ==0                                                                                ' If the number received is zero the SD card drive has mounted
   lcd.str(string("Mounted."))
   sdfat.opendir
else
   lcd.gotoxy(2,1)                                                                      ' If the number received is not zero we can tell the user of the problem
   if r == -20
      lcd.str(string("NOT FAT16 DISK"))
   elseif r == -21
      lcd.str(string("BAD BYTES/SCTR"))         
   lcd.str(string("Failed!"))

   
'3.5) Start switches in a new cog
'-------------------------------- 
cognew(switch, @stack1)                                                                  ' Start switch monitoring 


'4) Start doing things.
'===================== 
callCls
lcd.str(string("Awaiting"))                                                             ' Put "Awaiting Connection..." on the LCD until comms are established
lcd.gotoxy(2,1)
lcd.str(string("Connection..."))
 
lcdshow:= No                                                                            ' Turn off the echoing of RS232 data on the LCD screen.
lcdMode := No                                                                           ' We expect the data received to be a decimal    
                     
callATZ                                                                                 ' Send an ATZ Reset command
lcdshow:= No                                                                            ' We will NOT show it on the LCD
 
callCls                                                                                 ' Tell the user some handshaking is happening
lcd.str(String("Handshaking..."))
    
'GetPID data
 
 
repeat                                                                                 ' We are going to keep doing this FOREVER!
    if rec == 0
      lcdshow :=1        
      case Mode                                                                        ' Call the appropriate subroutine based upon the mode
        1:
          callCoolant         
        2:  
          callLambda   
        3:
          callLoad   
        4:
          callAFM   
        5:
          callMap   
        6:
          callRPM(1)
        7:
          callSpeed
        8:
          callAdvance
        9:
          callIAT
        10:
           callThrottle
        '11:
        '   callAcc
        
 
      !outa[LED2_PIN]                                                                   ' Toggle the LED1
      
    elseif rec == 1                                                                     ' Recording mode 1 - record OBD data
        bytemove(@dataOut,num.dec(Cnt),32)                                              ' Record the system counter to the SD card.
        sdfat.pwrite(@dataOut,32)
        sdfat.pputc(44)     
     
        lcdshow:=0       
        lcd.gotoxy(1,1)                                                                 
        callRPM(0)                                                                        ' Request, convert and display RPM
        callWriteStuff                                                                  ' Write the result to the SD card 
        
        callLoad                                                                        ' Request, convert and display engine load
        callWriteStuff                                                                  ' Write the result to the SD card        
     
        callAFM                                                                         ' Request, convert and display airflow
        lcd.gotoxy(2,1)                                                                 
        callWriteStuff                                                                  ' Write the result to the SD card     
        
        callThrottle                                                                    ' Request, convert and display throttle position    
        callWriteStuff                                                                  ' Write the result to the SD card
        
        sdfat.pputc(13)                                                                 ' Write a new line to the SD card
     
        lcd.gotoXY(2,16)
        lcd.str(string("R"))                                                            ' Display an 'R' character (for 'recording')
        !outa[LED1_PIN]                                                                 ' Toggle the 'record' LED
 
    {{else                                                                                ' Recrding mode 2 - record acceleration data
      callCls
      lcd.str(string("Recording "))
      
      bytemove(@dataOut,num.dec(Cnt),32)                                                ' Record the system counter to the SD card.
         sdfat.pwrite(@dataOut,32)
         sdfat.pputc(comma)
       callAxRec(xAxis)                                                                 ' Obtain the X axis acceleration
       callWriteFloat                                                                   ' Write the result to the SD card
       callAxRec(yAxis)                                                                 ' Obtain the Y axis acceleration
       callWriteFloat                                                                   ' Write the result to the SD card
       callAxRec(zAxis)                                                                 ' Obtain the Z axis acceleration
       callWriteFloat                                                                   ' Write the result to the SD card
       sdfat.pputc(13)                                                                  ' Write a new line to the SD card
       waitcnt(20_000_000 + cnt)}}
       !outa[LED1_PIN]      
pub callAT
  {{callAT sends a string to the ELM327 that is "AT ". This is basic to many of the commands that need to be sent to the ELM327, so is a useful building block that can be re-used many times.}}
  uart.tx(65)                                                                           ' Send character 'A'
  uart.tx(84)                                                                           ' Send character 'T'
  uart.tx(32)                                                                           ' Send a space character  
pub callATZ
  {{callATZ sends a string to the ELM327 that is "AT Z". This resets the ELM327 and results in its descriptor being returned. We don't check for the descriptor being as expected.}}
   txDisp:= string("Reset:")
   callAT                                                                               ' Send characters "AT "
   uart.tx(90)                                                                          ' Send character 'Z'            
   uart.tx(13)                                                                          ' Send a carriage return
   callWaitRx                                                                           ' Wait for, and process, the returned data
pub callCoolant 
   {{callCoolant requests PID05 data - engine coolant temperature. The returned data is converted from the hex to decimal and displayed to the user on the LCD (or recorded).}}
   callPID01(48, 53)                                                                    ' Send the request                                                                    
   callLCDShow (string("Coolant temp"),string(" C"), AnsaA - 40)                        ' Show the data on the LCD
pub callLoad     
   {{callLoad requests PID04 data - engine load. The returned data is converted from the hex to decimal and displayed to the user on the LCD (or recorded).}}
   callPID01(48, 52)                                                                    ' Send the request
   callLCDShow(string("Engine Load"), string(" %"), AnsaA * 100 / 255 )                 ' Show the data on the LCD   
   if rec == 0                                                                          ' If there is no recording going on add a bargraph
      callBarGraph(6, 2, 0, AnsaA*50/300)
   AnsaA := AnsaA * 100/255 
pub callMAP     
   {{callMAP requests PID0B data - manifold pressure. The returned data is converted from the hex to decimal and displayed to the user on the LCD (or recorded).}}
   callPID01(48, 66)                                                                    ' Send the request
   callLCDShow (string("MAP px"),string("kPa"), AnsaA * 100 / 255)                      ' Show the data on the LCD
   callBarGraph (6, 2, 0, AnsaA*50/200) 
pub callRPM(f)             
   {{callRPM requests PID0C data - engine RPM. The returned data is converted from the hex to decimal and displayed to the user on the LCD (or recorded).}}
   callPID01(48, 67)
   callLCDShow(string("Engine RPM"), string(" "), ((AnsaB  * 256) + AnsaA)/4)           ' Show the data on the LCD   
   if f == 1
     callBarGraph(7, 2, 0, ((AnsaB  * 256) + AnsaA)/400)
   else
     AnsaA := ((AnsaB  * 256) + AnsaA)/4  
pub callSpeed      
   {{callRPM requests PID0D data - Vehicle speed. The returned data is converted from the hex to decimal and displayed to the user on the LCD (or recorded).}}
   callPID01(48, 68)                                                                    ' Send the request
   callLCDShow (string("Speed"), string(" km/h"), AnsaA)                                ' Show the data on the LCD
pub callAdvance    
   {{callRPM requests PID0E data - Ignition advance. The returned data is converted from the hex to decimal and displayed to the user on the LCD (or recorded).}}
   callPID01(48, 69)                                                                    ' Send the request
   callLCDShow(string("Advance"), string(" Deg"),AnsaA / 2 - 64 )                       ' Show the data on the LCD
pub callIAT    
   {{callRPM requests PID0F data - Inlet air temperaturee. The returned data is converted from the hex to decimal and displayed to the user on the LCD (or recorded).}}
   callPID01(48, 70)                                                                    ' Send the request
   callLCDShow (string("Inlet Temp"), string(" C"), AnsaA - 40)                         ' Show the data on the LCD
pub callAFM    
   {{callRPM requests PID10 data - Airflow rate. The returned data is converted from the hex to decimal and displayed to the user on the LCD (or recorded).}}
   callPID01(49, 48)                                                                    ' Send the request
   callLCDShow(string("Airflow"), string(" g/s"), ((AnsaB * 256)+ AnsaA)/100)           ' Show the data on the LCD
   AnsaA := ((AnsaB * 256)+ AnsaA)/100
pub callThrottle
   {{callThrottle requests PID11 data - Throttle position. The returned data is converted from the hex to decimal and displayed to the user on the LCD (or recorded).}}
   callPID01(49, 49)                                                                    ' Send the request
   callLCDShow(string("Throttle pos"), units := string(" %"), AnsaA * 100 / 255)        ' Show the data on the LCD
   if rec == 0                                                                          ' If there is no recording going on add a bargraph
        callBarGraph(6, 2, 0, AnsaA*50/300)                                             ' Send the value (in 1/8's of a character width)
   AnsaA := AnsaA * 100 / 255
pub callLambda
   {{ This procedure sends "01 14" to the ELM327 unit to request the lambda value. This is Mode 1, PID 14. The 01 is sent via the 
      callPID01 procedure.}}   
   callPID01(49, 52)                                                                    ' Send the request
   if rec == 0                                                                          ' If there is no recording going on add a bargraph
     callCls                                                                            ' Clear the display
     callBarGraph(1, 2, 0,((AnsaB*80)/140))
     AnsaA := AnsaB     
     if AnsaA < 45
        lcd.gotoXY(1,13)                                                                ' If the mixture is lean display some text
        lcd.str(string("Lean"))
     elseif AnsaA > 80
        lcd.gotoXY(1,13) 
        lcd.str(string("Rich"))
     else
        lcd.gotoXY(1,13) 
        lcd.str(string(" "))                                                    ' If the mixture is rich display some text
pub callWaitRx
   {{callWaitRx receives a string of characters from the ELM327 unit. These are loaded into a buffer until the end of the data is detected. The data in the buffer is then available for processing and analysis.}}
 
   longfill(dataIn, 100, 0)                                                             ' Empty the dataIn variable
   longfill(dataOut, 100, 0)                                                            ' Empty the dataOut variable
   ct := 0                                                                              ' Reset the counter for the number of characters received
   'ctOut := 0                                         
   if lcdshow == Yes                                                                    ' We may want to echo received data to the LCD.
        callCls                                                                         ' Clear the LCD
        lcd.str(txDisp)                                                                 ' Show the data on the first row
        lcd.gotoxy(2,1)                                                                 ' Move the cursor to the next line
   repeat while Rx <>= 62                                                               ' Continue to collect data until the end of the data (ASCII 62)     
     Rx := uart.Rx                                                                      ' Get character from Rx Buffer
     dataIn[ct++] := Rx                                                                 ' Place the received character in the dataIn variable
     'text.out(dataIn[ct-1])                                                            ' Echo characters received to screen
   Rx:=42                                                                               ' Set 'Rx' to a value that is NOT 62, otherwise it won't                                                                                        ' go through the 'repeat while' loop on next encounter                   uart.rxflush                                                                   ' Flush the Rx buffer
pub callhexToDec (Hex1, Hex2, Hex3, Hex4) | V1, V2, V3, V4
{{callHexToDec converts fours ASCII-coded characters into decimal values}}
V1 := 0                                                                                 ' Set the value variables to 0
V2 := 0
V3 := 0
V4 := 0
 
if Hex1 > 64                                                                            ' Convert the ASCII code to decimal - character 1
   V1 := Hex1 - 55
else
   V1 := Hex1 - 48
 
if Hex2 > 64                                                                            ' Convert the ASCII code to decimal - character 2
   V2 := Hex2 - 55
else
   V2 := Hex2 - 48
 
if Hex3 > 64                                                                            ' Convert the ASCII code to decimal - character 3
   V3 := Hex3 - 55
else
   V3 := Hex3 - 48
 
if Hex4 > 64                                                                            ' Convert the ASCII code to decimal - character 4
   V4 := Hex4 - 55
else
   V4 := Hex4 - 48
 
AnsaA := (V3 * 16) + V4                                                                 ' Convert into a decimal ensuring place-value is respected
AnsaB := (V1 * 16) + V2                                                                 ' Convert into a decimal ensuring place-value is respected

pub callLCDShow(cTitle, cUnits, v)
{{callLCDShow is a procedure that displays the OBD live data to the user. Title text and units are included and it is able to display decimals and string values.}} 
if rec == No and err == No                                                              ' Only display if recording is not in progress and there are no errors
   callCls                                                                              ' Clear the LCD
   lcd.str(cTitle)                                                                      ' Show the title on the first row
   callPutC(58)                                                                         ' Add a colon
   lcd.gotoxy(2,1)                                                                      ' Move the cursor to the next row
 
   if lcdMode == 0                                                                      ' Process the data depending on the value of lcdMode
     lcd.dec(v)                                                                     ' Its a decimal
   else
     lcd.str(v)                                                                     ' Its a string
   lcd.str(cUnits)                                                                       ' Display the units
pub switch
{{switch deals with the state of the selection switches. It runs parallel to all other activity in a dedicated cog and provides outputs via global variables that indicate whether each switch is pressed or not.}}
index1~                                                                                 ' index is initially zero
repeat until ((ina[SW1_PIN] == notSelected) and (ina[SW2_PIN] == notSelected) and (ina[SW3_PIN] == notSelected))    ' wait until all three buttons are confirmed as NOT SELECTED
repeat
   repeat until ((ina[SW1_PIN] == selected) or (ina[SW2_PIN] == selected) or (ina[SW3_PIN] == selected)) ' wait for any to be on
   index1++
 
   if (ina[SW1_PIN] == selected)                                                        ' If switch 1 is selected update the flag and increment mode
     mode++
     sw1 := Selected
     if mode == (maxmode + 1)                                                           ' If we exceed the maximum number of choices roll-over back to 1
        mode := 1
 
   if (ina[SW2_PIN] == selected)                                                        ' If switch 2 is selected update the flag and decrement mode
     mode--
     sw2 := Selected
     if mode == 0                                                                       ' Roll-over to maximum
        mode := maxmode
 
   if (ina[SW3_PIN] == selected)                                                        ' If switch 3 is selected change recording mode                                         
     if rec == 0
       rec := 1
       sdfat.popen(string("Data3.csv"), "a")
       
     elseif rec ==1
       rec := 2

     else                                                                               ' If we are recording accelerometer data close the sd file and reboot.
       rec := 0
       sdfat.pclose
       
       callDoWait
       waitcnt(80_000_000 + cnt)
       reboot
   repeat until ((ina[SW1_PIN] == notSelected) and (ina[SW2_PIN] == notSelected) and (ina[SW3_PIN] == notSelected)) ' Wait for all switches to be selected off 
      waitcnt(clkfreq/index1 + cnt)
sw1:=0                                                                                  ' Reset the sw1 flag
sw2:=0                                                                                  ' Reset the sw2 flag

pub callPID01(cChar1, cChar2)
  {{callPID01 is a procedure that can call any Mode 1 PID, wait for the results and load the data into the Hexn variables for processing into the answer.}}

  uart.tx(48)                                                                           ' Send '0'
  uart.tx(49)                                                                           ' Send '1'
  uart.tx(32)                                                                           ' Send a space
  uart.tx(cChar1)                                                                       ' Send the first character of the PID ID
  uart.tx(cChar2)                                                                       ' Send the second character of the PID ID
  uart.tx(13)                                                                           ' Send a carriage return
  lcdshow := No                                                                         ' Don't echo the data           
  callWaitRx                                                                            ' Receive the data from the ELM327
  H3 := dataIn[ct - 6]                                                                  ' Populate the Hex3 variable with the penultimate character of the response.
  H4 := dataIn[ct - 5]                                                                  ' Populate the Hex4 variable with the last character of response.

  if H3 == 84 and H4 == 65                                                               ' Capture 'NO DATA' error
    callRetry(string("NO DATA"))         
  elseif H3 == 82 and H4 == 79    
    callRetry(string("DATA ERR"))                                                        ' Capture 'DATA ERROR' error
  elseif H4 == 63
    callRetry(string("WHAT?"))                                                           ' Capture error where request has been rejected by ELM327
  else
     err := 0  

  callhexToDec(dataIn[ct - 9], dataIn[ct - 8], dataIn[ct - 6], dataIn[ct - 5])            ' Convert the data to decimal.   
pub callDoWait
   {{callDoWait causes a small delay to be effected. This can be called from anywhere where a small delay is required, i.e., for timing problems.}}
   waitcnt(40_000_000 + cnt)
pub callWriteStuff
  {{callWriteStuff contains a small amount of code that writes the answer from OBD data calculations to the SD card.}}
  lcd.dec(AnsaA)                                                                        ' Put the data on the display
  bytemove(@dataOut,num.dec(AnsaA),5)                                                   ' Copy the answer to the buffer as a decimal
  sdfat.pwrite(@dataOut,5)                                                              ' Write the buffer (5 characters) to the SD file
  sdfat.pputc(comma)                                                                    ' Write the delimiter to the file
  lcd.str(String("  "))                                                                 ' Add a space character to the display.
pub callWriteFloat
  {{callWriteFloat contains a small amount of code that writes the floating-point answers from accelerometer data to the SD card.}}
  bytemove(@dataOut,num.dec(aq),5)                                                      ' Copy the answer to the buffer as a decimal
  sdfat.pwrite(@dataOut,5)                                                              ' Write the buffer (5 characters) to the SD file
  sdfat.pputc(comma)                                                                    ' Write the delimiter to the file
  lcd.str(String("  "))                                                                 ' Add a space character to the display.
pub callRetry(Msg)
{{callRetry is a procedure that is called when a problem with the data comms is reported by the ELM327}}
if doErr == Yes                                                                         ' Only worry if we are going to deal with errors
  callCls                                                                               ' Clear the display
  lcd.str(Msg)                                                                          ' Display the warning message
  err := 1
  callDoWait                                                                            ' Wait for a moment
  lcd.gotoxy(2,1)                                                                       ' Move the curso to row 2
  lcd.str(string("Retrying..."))                                                        ' Add some text
  callDoWait                                                                            ' Wait for a moment

  
pub callAcc
{{callAcc is a procedure that displays live acceleration data with a bargraph display for lateral acceleration.}}
{{    callCls                                                                             ' Clear the display
    'callAx(1, 1, zAxis)
    callAx(1, 1, yAxis)                                                                 ' Place y-axis data in row 1, column 1 of the display    
    callAx(1, 9, xAxis)                                                                 ' Place x-axis data in row 1, column 9 of the display
    
    aq:=flt.fmul(aq, 30)                                                                ' Scale the data 
    callPutC(254)                                                                       ' Initiate the bargraph
    callPutC(124)
    if aqin >2048                                                                       ' Depending on whether the acceleration is to l or r
      callPutC(7)                                                                       '...select column 7, or
    else
      callPutC(8)                                                                       '...select column 8, then
    callPutC(2)              
    if aqin >2048
      callPutC(1)                                                                       '...direction is to the right, or
    else
      callPutC(0)              
    callPutC(aq)                                                                        '...direction is to the left.
     waitcnt(10_000_000 + cnt)    
{pub callAx (co, ro, v) 
{{callAx receives column and row reference together with the axis identifier, inputs data from the relevant ADC channel and displays the data.}} 
   {{ lcd.gotoxy(co, ro)                                                                  ' Place the cursor
    aqin:= ADC.in( v)                                                                   ' Get the value from the ADC channel.
    if aqin < 2048
       lcd.str(string("-"))                                                             ' If negative, add a minus sign.
       aq:=2048 - aqin
    else
       aq:=aqin-2048
    aq:=flt.fdiv(aq,374)                                                                ' Scale to 1 'g'

    lcd.str(fltstr.floattoformat(aq,5,3))                                               ' Convert to a string
    lcd.str(string(" "))  }}                                                              ' Display he data
{{pub callAxRec(v)    
{callAxRec gets the data from the ADC for the channel requested}}
   {{ aq:= ADC.in( v)   }}

pub callBarGraph(Col, Row, Dir, V)
      callPutC(254)                                                                     ' Send ASCII 254
      callPutC(124)                                                                     ' Send ASCII 124
      callPutC(col)                                                                       ' Send column number
      callPutC(Row)                                                                       ' Send row number
      callPutC(Dir)                                                                       ' Send direction (0 = to the right)
      callPutC(V)                                                            ' Send the value (in 1/8's of a character width)
pub callPutC(x)
  lcd.putc(x)
pub callCls
  lcd.cls
